@@ -39,7 +39,11 @@
 #undef KDE_ICONDIR
 #undef KDE_MINI_ICONDIR
 
+/* Location of KDE documentation */
+#undef KDE_DOCDIR
+
 #undef HAVE_ZVT_TERM_RESET
+#undef HAVE_ZVT_DEL_IS_DEL
 
 #undef HAVE_HOSTS_ACCESS
 
