@@ -9,8 +9,9 @@
  * Feel free to implement new look and feels :-)
  */
 
+#include "config.h"
+
 #include <stdio.h>
-#include <config.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -23,6 +24,15 @@
 
 #include "clock.h"
 
+#define INTERNETSECOND (864/4)
+
+/* values for selection info */
+enum {
+  TARGET_STRING,
+  TARGET_TEXT,
+  TARGET_COMPOUND_TEXT,
+};
+
 typedef struct _ClockData ClockData;
 typedef void (*ClockUpdateFunc) (ClockData *, time_t);
 struct _ClockData {
@@ -33,18 +43,21 @@ struct _ClockData {
 	int hourformat;
         gboolean showdate;
 	gboolean unixtime;
+	gboolean internettime;
 	gboolean gmt_time;
 	gboolean showtooltip;
 	ClockUpdateFunc update_func;
 	PanelOrientType orient;
 	int size;
+	char *copied_string;
 
 	GtkWidget *props;
 	int prop_hourformat;
-        int prop_showdate;
-	int prop_unixtime;
-	int prop_gmt_time;
-	int prop_showtooltip;
+        gboolean prop_showdate;
+	gboolean prop_unixtime;
+	gboolean prop_internettime;
+	gboolean prop_gmt_time;
+	gboolean prop_showtooltip;
 };
 
 typedef struct {
@@ -82,6 +95,14 @@ clock_timeout_callback(gpointer data)
 						      cd);
 			return FALSE;
 		}
+	} else if(cd->internettime) {
+		if (cd->timeouttime != INTERNETSECOND) {
+			cd->timeouttime = INTERNETSECOND;
+			cd->timeout = gtk_timeout_add(cd->timeouttime,
+						      clock_timeout_callback,
+						      cd);
+			return FALSE;
+		}	   
 	} else {
 		if (cd->timeouttime != 36000) {
 			cd->timeouttime = 36000;
@@ -106,6 +127,7 @@ applet_save_session(GtkWidget * w,
 	gnome_config_set_int("clock/hourformat", cd->hourformat);
 	gnome_config_set_int("clock/showdate", cd->showdate);
 	gnome_config_set_int("clock/unixtime", cd->unixtime);
+	gnome_config_set_int("clock/internettime", cd->internettime);
 	gnome_config_set_int("clock/showtooltip", cd->showtooltip);
 	gnome_config_set_int("clock/gmt_time", cd->gmt_time);
 	gnome_config_pop_prefix();
@@ -115,6 +137,19 @@ applet_save_session(GtkWidget * w,
 	return FALSE;
 }
 
+static float
+get_itime (time_t current_time)
+{
+	struct tm *tm;
+	float itime;
+	time_t bmt;
+
+	bmt = current_time + 3600;
+	tm = gmtime(&bmt);
+	itime = (tm->tm_hour*60+tm->tm_min+tm->tm_sec/60.0)*1000.0/1440.0;
+
+	return itime;
+}
 
 static void
 computer_clock_update_func(ClockData * cd, time_t current_time)
@@ -122,7 +157,7 @@ computer_clock_update_func(ClockData * cd, time_t current_time)
 	ComputerClock *cc;
 	struct tm *tm;
 	GString *gs = g_string_new("");
-	char date[20], hour[20], tooltip[30];
+	char date[256], hour[256], tooltip[256];
 
 	cc = gtk_object_get_user_data(GTK_OBJECT(cd->clockw));
 
@@ -134,46 +169,50 @@ computer_clock_update_func(ClockData * cd, time_t current_time)
 	if (cd->unixtime) {
 		if ((cd->orient == ORIENT_LEFT || cd->orient == ORIENT_RIGHT) &&
 		    cd->size >= PIXEL_SIZE_STANDARD) {
-			g_snprintf(hour,20,"%lu\n%05lu",
+			g_snprintf(hour,sizeof(hour),"%lu\n%05lu",
 				   (unsigned long)(current_time/100000L),
 				   (unsigned long)(current_time%100000L));
 		} else {
-			g_snprintf(hour,20,"%lu",(unsigned long)current_time);
+			g_snprintf(hour,sizeof(hour),"%lu",(unsigned long)current_time);
 		} 
 		g_string_append(gs,hour);
+	} else if (cd->internettime) {
+		float itime = get_itime (current_time);
+		g_snprintf(hour,sizeof(hour),"@%3.2f",itime);
+		g_string_append(gs,hour);
 	} else if (cd->hourformat == 12) {
 		/* This format string is used, to display the actual time in
 		   12 hour format.  */
 		if ((cd->orient == ORIENT_LEFT || cd->orient == ORIENT_RIGHT) &&
 		    cd->size >= PIXEL_SIZE_STANDARD) {
-			if (strftime(hour, 20, _("%I:%M\n%p"), tm) == 20)
-				hour[19] = '\0';
+			if (strftime(hour, sizeof (hour), _("%I:%M\n%p"), tm) <= 0)
+				strcpy (hour, "???");
 		} else {
-			if (strftime(hour, 20, _("%I:%M %p"), tm) == 20)
-				hour[19] = '\0';
+			if (strftime(hour, sizeof (hour), _("%I:%M %p"), tm) <= 0)
+				strcpy (hour, "???");
 		} 
 		g_string_append(gs,hour);
 	} else if (cd->hourformat == 24) {
 		/* This format string is used, to display the actual time in
 		   24 hour format.  */
-		if (strftime(hour, 20, _("%H:%M"), tm) == 20)
-			hour[19] = '\0';
+		if (strftime(hour, sizeof (hour), _("%H:%M"), tm) <= 0)
+			strcpy (hour, "???");
 		g_string_append(gs,hour);
 	}
 
-	if (cd->showdate && !cd->unixtime) {
+	if (cd->showdate && !cd->unixtime && !cd->internettime) {
 		if ((cd->orient == ORIENT_LEFT || cd->orient == ORIENT_RIGHT) &&
 		    cd->size >= PIXEL_SIZE_STANDARD) {
 			/* This format string is used, to display the actual day,
 			   when showing a vertical panel.  For an explanation of
 			   this format string type 'man strftime'.  */
-			if (strftime(date, 20, _("%a\n%b %d"), tm) == 20)
-				date[19] = '\0';
+			if (strftime(date, sizeof (date), _("%a\n%b %d"), tm) <= 0)
+				strcpy (date, "???");
 		} else {
 			/* This format string is used, to display the actual day,
 			   when showing a horizontal panel.  */
-			if (strftime(date, 20, _("%a %b %d"), tm) == 20)
-				date[19] = '\0';
+			if (strftime(date, sizeof (date), _("%a %b %d"), tm) <= 0)
+				strcpy (date, "???");
 		}
 		if(cd->size < PIXEL_SIZE_STANDARD)
 			g_string_append_c(gs,' ');
@@ -183,9 +222,9 @@ computer_clock_update_func(ClockData * cd, time_t current_time)
 	}
 	
 	/* Set the applets tooltip */
-	if (cd->showtooltip && !cd->unixtime) {
-		if (strftime(tooltip, 30, _("%A, %B %d"), tm) == 30)
-			date[29] = '\0';
+	if (cd->showtooltip && !cd->unixtime && !cd->internettime) {
+		if (strftime(tooltip, sizeof (tooltip), _("%A, %B %d"), tm) <= 0)
+			strcpy (tooltip, "???");
 		applet_widget_set_tooltip(APPLET_WIDGET(cd->applet), tooltip);
 	}
 	
@@ -218,7 +257,7 @@ create_computer_clock_widget(GtkWidget ** clock, ClockUpdateFunc * update_func)
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(frame);
 
-	cc = g_new(ComputerClock, 1);
+	cc = g_new0 (ComputerClock, 1);
 
 	cc->align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
 	gtk_container_set_border_width(GTK_CONTAINER(cc->align), 3);
@@ -247,9 +286,15 @@ static void
 destroy_clock(GtkWidget * widget, gpointer data)
 {
 	ClockData *cd = data;
-	gtk_timeout_remove(cd->timeout);
+
+	if (cd->timeout > 0)
+		gtk_timeout_remove (cd->timeout);
+	cd->timeout = 0;
+
 	if(cd->props)
 		gtk_widget_destroy(cd->props);
+	cd->props = NULL;
+
 	g_free(cd);
 }
 
@@ -286,6 +331,8 @@ create_clock_widget(ClockData *cd, GtkWidget * applet)
     
 	if(cd->unixtime)
 		cd->timeouttime = 1000;
+        else if(cd->internettime)
+                cd->timeouttime = INTERNETSECOND;
 	else
 		cd->timeouttime = 36000-(tm->tm_sec*600);
 	cd->timeout = gtk_timeout_add(cd->timeouttime,
@@ -323,11 +370,134 @@ applet_change_pixel_size(GtkWidget * w, int size, gpointer data)
 	(*cd->update_func) (cd, current_time);
 }
 
+static void
+copy_time (AppletWidget *applet, gpointer data)
+{
+	ClockData *cd = data;
+	time_t current_time = time (NULL);
+	struct tm *tm;
+	char string[256];
+
+	if (cd->gmt_time)
+		tm = gmtime(&current_time);
+	else
+		tm = localtime(&current_time);
+
+	if (cd->unixtime) {
+		g_snprintf (string, sizeof(string), "%lu",
+			    (unsigned long)current_time);
+	} else if (cd->internettime) {
+		float itime = get_itime (current_time);
+		g_snprintf (string, sizeof (string), "@%3.2f", itime);
+	} else if (cd->hourformat == 12) {
+		/* This format string is used, to display the actual time in
+		   12 hour format.  */
+		if (strftime(string, sizeof (string), _("%I:%M %p"), tm) <= 0)
+			strcpy (string, "???");
+	} else if (cd->hourformat == 24) {
+		/* This format string is used, to display the actual time in
+		   24 hour format.  */
+		if (strftime(string, sizeof (string), _("%H:%M"), tm) <= 0)
+			strcpy (string, "???");
+	}
+
+	g_free (cd->copied_string);
+	cd->copied_string = g_strdup (string);
+
+	gtk_selection_owner_set (GTK_WIDGET (applet),
+				 GDK_SELECTION_PRIMARY,
+				 GDK_CURRENT_TIME);
+}
+
+static void
+copy_date (AppletWidget *applet, gpointer data)
+{
+	ClockData *cd = data;
+	time_t current_time = time (NULL);
+	struct tm *tm;
+	char string[256];
+
+	if (cd->gmt_time)
+		tm = gmtime(&current_time);
+	else
+		tm = localtime(&current_time);
+
+	if (strftime(string, sizeof (string), _("%A, %B %d %Y"), tm) <= 0)
+		strcpy (string, "???");
+	
+	g_free (cd->copied_string);
+	cd->copied_string = g_strdup (string);
+
+	gtk_selection_owner_set (GTK_WIDGET (applet),
+				 GDK_SELECTION_PRIMARY,
+				 GDK_CURRENT_TIME);
+}
+
+/* current timestamp */
+static void
+copy_timestamp (AppletWidget *applet, gpointer data)
+{
+	ClockData *cd = data;
+	time_t current_time = time (NULL);
+	struct tm *tm;
+	char string[256];
+
+	tm = localtime(&current_time);
+
+	if (strftime(string, sizeof (string),
+		     /* RFC822 conformant date, likely not different for other
+		      * locales I don't think */
+		     _("%a, %d  %b  %Y %H:%M:%S %z"), tm) <= 0)
+		strcpy (string, "???");
+	
+	g_free (cd->copied_string);
+	cd->copied_string = g_strdup (string);
+
+	gtk_selection_owner_set (GTK_WIDGET (applet),
+				 GDK_SELECTION_PRIMARY,
+				 GDK_CURRENT_TIME);
+}
+
+static void
+selection_handle (GtkWidget *widget, 
+		  GtkSelectionData *selection_data,
+		  int info, int time,
+		  gpointer data)
+{
+	ClockData *cd = data;
+
+	if (info == TARGET_STRING) {
+		gtk_selection_data_set (selection_data,
+					GDK_SELECTION_TYPE_STRING,
+					8 * sizeof(gchar),
+					(guchar *)cd->copied_string,
+					strlen (cd->copied_string));
+	} else if ((info == TARGET_TEXT) || (info == TARGET_COMPOUND_TEXT)) {
+		guchar *text;
+		GdkAtom encoding;
+		gint format;
+		gint new_length;
+
+		gdk_string_to_compound_text (cd->copied_string,
+					     &encoding, &format,
+					     &text, &new_length);
+		gtk_selection_data_set (selection_data, encoding, format,
+					text, new_length);
+		gdk_free_compound_text (text);
+	}
+}
+
 GtkWidget *
 make_clock_applet(const gchar * goad_id)
 {
 	ClockData *cd;
 	GtkWidget *applet;
+	GtkTargetEntry targets[] = {
+		{ "STRING", TARGET_STRING },
+		{ "TEXT",   TARGET_TEXT }, 
+		{ "COMPOUND_TEXT", TARGET_COMPOUND_TEXT }
+	};
+	gint n_targets = sizeof(targets) / sizeof(targets[0]);
 
 	applet = applet_widget_new(goad_id);
 	if (!applet) {
@@ -335,7 +505,9 @@ make_clock_applet(const gchar * goad_id)
 		return NULL;
 	}
 
-	cd = g_new(ClockData, 1);
+	cd = g_new0 (ClockData, 1);
+
+	cd->copied_string = NULL;
 
 	gnome_config_push_prefix(APPLET_WIDGET(applet)->privcfgpath);
 
@@ -375,6 +547,7 @@ make_clock_applet(const gchar * goad_id)
 		cd->showdate = gnome_config_get_int("clock/showdate=1");
 
 	cd->unixtime = gnome_config_get_int("clock/unixtime=0");
+	cd->internettime = gnome_config_get_int("clock/internettime=0");
 	/* if on a small screen show tooltip with date by default */
 	if(gdk_screen_width()<=800)
 		cd->showtooltip = gnome_config_get_int("clock/showtooltip=1");
@@ -412,6 +585,27 @@ make_clock_applet(const gchar * goad_id)
 					      clock_properties,
 					      cd);
 
+	applet_widget_register_stock_callback (APPLET_WIDGET (applet),
+					       "copy_time",
+					       GNOME_STOCK_MENU_COPY,
+					       _("Copy time to selection"),
+					       copy_time,
+					       cd);
+
+	applet_widget_register_stock_callback (APPLET_WIDGET (applet),
+					       "copy_date",
+					       GNOME_STOCK_MENU_COPY,
+					       _("Copy date to selection"),
+					       copy_date,
+					       cd);
+
+	applet_widget_register_stock_callback (APPLET_WIDGET (applet),
+					       "copy_timestamp",
+					       GNOME_STOCK_MENU_COPY,
+					       _("Copy timestamp to selection"),
+					       copy_timestamp,
+					       cd);
+
 	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
 					      "help",
 					      GNOME_STOCK_PIXMAP_HELP,
@@ -425,6 +619,13 @@ make_clock_applet(const gchar * goad_id)
 					      _("About..."),
 					      clock_about,
 					      NULL);
+
+	gtk_selection_add_targets (GTK_WIDGET (applet), GDK_SELECTION_PRIMARY,
+				   targets, n_targets);
+
+        gtk_signal_connect (GTK_OBJECT (applet), "selection_get",
+			    GTK_SIGNAL_FUNC (selection_handle), cd);
+
 	return applet;
 }
 
@@ -438,6 +639,7 @@ apply_properties(GtkWidget * widget, gint button_num, gpointer data)
 	cd->hourformat = cd->prop_hourformat;
 	cd->showdate = cd->prop_showdate;
 	cd->unixtime = cd->prop_unixtime;
+   	cd->internettime = cd->prop_internettime;
 	cd->gmt_time = cd->prop_gmt_time;
 	cd->showtooltip = cd->prop_showtooltip;
 
@@ -455,12 +657,14 @@ apply_properties(GtkWidget * widget, gint button_num, gpointer data)
 
 	if(cd->unixtime)
 		cd->timeouttime = 1000;
+        else if(cd->internettime)
+                cd->timeouttime = INTERNETSECOND;
 	else
 		cd->timeouttime = 36000-(tm->tm_sec*600);
 	cd->timeout = gtk_timeout_add(cd->timeouttime,
 				      clock_timeout_callback,
 				      cd);		      
-	if(!cd->showtooltip || cd->unixtime)
+	if(!cd->showtooltip || cd->unixtime || cd->internettime)
   		applet_widget_set_tooltip(APPLET_WIDGET(cd->applet), "");
 				      
 /* gtk_widget_queue_resize (cd->clockw);*/
@@ -500,6 +704,15 @@ set_datasensitive_cb(GtkWidget * w, GtkWidget *wid)
 	gtk_widget_set_sensitive(wid,!(GTK_TOGGLE_BUTTON(w)->active));
 }
 
+static void
+set_internettime_cb(GtkWidget * w, gpointer data)
+{
+	ClockData *cd = data;
+
+	cd->prop_internettime = GTK_TOGGLE_BUTTON(w)->active;
+	gnome_property_box_changed (GNOME_PROPERTY_BOX (cd->props));
+}
+
 static void
 set_unixtime_cb(GtkWidget * w, gpointer data)
 {
@@ -539,17 +752,21 @@ clock_properties(AppletWidget * applet, gpointer data)
 	GtkWidget *showdate;
 	GtkWidget *showtooltip;
 	GtkWidget *unixtime;
+	GtkWidget *internettime;
 	GtkWidget *use_gmt_time;
 	ClockData *cd = data;
 
 
-	if(cd->props) {
-		gdk_window_raise(cd->props->window);
+	if (cd->props != NULL) {
+		gtk_widget_show_now (cd->props);
+		gdk_window_raise (cd->props->window);
 		return;
 	}
+
 	cd->props = gnome_property_box_new();
 	gtk_window_set_title (GTK_WINDOW (cd->props),
 			      _("Clock properties"));
+	gtk_window_set_wmclass (GTK_WINDOW (cd->props), "clock", "Clock");
 	gnome_window_icon_set_from_file (GTK_WINDOW (cd->props),
 					 GNOME_ICONDIR"/gnome-clock.png");
 
@@ -657,7 +874,11 @@ clock_properties(AppletWidget * applet, gpointer data)
 	gtk_box_pack_start_defaults(GTK_BOX(vbox), unixtime);
 	gtk_widget_show(unixtime);
 
-	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
+        internettime = gtk_check_button_new_with_label(_("Internet time"));
+	gtk_box_pack_start_defaults(GTK_BOX(vbox), internettime);
+	gtk_widget_show(internettime);
+   
+        gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
 			   (GtkSignalFunc) set_datasensitive_cb,
 			   hour_frame);
 	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
@@ -669,14 +890,43 @@ clock_properties(AppletWidget * applet, gpointer data)
 	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
 			   (GtkSignalFunc) set_datasensitive_cb,
 			   use_gmt_time);
+	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   internettime);
 			   
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(unixtime),
 				     cd->unixtime);
 	cd->prop_unixtime = cd->unixtime;
-	
+
+
+/* internet time */
+	gtk_signal_connect(GTK_OBJECT(internettime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   hour_frame);
+	gtk_signal_connect(GTK_OBJECT(internettime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   showdate);
+	gtk_signal_connect(GTK_OBJECT(internettime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   showtooltip);
+	gtk_signal_connect(GTK_OBJECT(internettime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   use_gmt_time);   
+	gtk_signal_connect(GTK_OBJECT(internettime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   unixtime);   
+   
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(internettime),
+				     cd->internettime);
+	cd->prop_internettime = cd->internettime;
+/* */
+   
 	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
 			   (GtkSignalFunc) set_unixtime_cb,
 			   data);
+	gtk_signal_connect(GTK_OBJECT(internettime), "toggled",
+			   (GtkSignalFunc) set_internettime_cb,
+			   data);
 
 	gtk_widget_show(hour_frame);
 	gtk_widget_show(hbox);
@@ -718,21 +968,23 @@ clock_about (AppletWidget *applet, gpointer data)
 		"Miguel de Icaza <miguel@kernel.org>",
 		"Federico Mena <quartic@gimp.org>",
 		"Stuart Parmenter <pavlov@innerx.net>",
+		"George Lebl <jirka@5z.com>",
 		NULL
 	};
 
 	if (about != NULL)
 	{
-		gdk_window_show (about->window);
+		gtk_widget_show_now (about);
 		gdk_window_raise (about->window);
 		return;
 	}
 	
 	about = gnome_about_new (_("Clock Applet"), "1.0",
-				 _("(c) 1998 the Free Software Foundation"),
+				 _("(c) 1998-2001 the Free Software Foundation"),
 				 authors,
 				 _("The clock applet gives your panel a lightweight and simple display of the date and time"),
 				 GNOME_ICONDIR"/gnome-clock.png");
+	gtk_window_set_wmclass (GTK_WINDOW (about), "clock", "Clock");
 	gnome_window_icon_set_from_file (GTK_WINDOW (about),
 					 GNOME_ICONDIR"/gnome-clock.png");
 	gtk_signal_connect (GTK_OBJECT(about), "destroy",
