@@ -18,9 +18,10 @@ static gchar *contributors[] = {
 	"Shawn T. Amundson",
 	"Erik Andersen",
 	"Jon Anhold",
-	"Timur I. Bakeyev",
-	/* for languages that can't display aacute, replace '' with 'a' */
-	N_("Szabolcs 'Shooby' Bn"),
+	N_("Timur I. Bakeyev"),
+	/* if your encoding allows it, use aacute (U00E1) 
+	 * for the 'a' of 'Ban' */
+	N_("Szabolcs 'Shooby' Ban"),
 	"Martin Baulig",
 	"Tom Bech",
 	"Andreas Beck",
@@ -32,7 +33,7 @@ static gchar *contributors[] = {
 	"Robert Bihlmeyer",
 	"Jonathan Blandford",
 	"Christopher Blizzard",
-	"Jerome Bolliet",
+	N_("Jerome Bolliet"),
 	"Andreas Bolsch",
 	"Dario Bressanini",
 	"Emmanuel Briot",
@@ -56,7 +57,9 @@ static gchar *contributors[] = {
 	"Dan Damian",
 	"Phil Dawes",
 	"Fatih Demir",
-	"Frederic Devernay",
+	/* if your encoding allows it, use eacute (U00E9) for the two
+	 * 'e' of 'Frederic' */
+	N_("Frederic Devernay"),
 	"Dietmar Maurer",
 	"Feico W. Dillema",
 	"Radek Doulik",
@@ -67,12 +70,16 @@ static gchar *contributors[] = {
 	"Gus Estrella",
 	"David Etherton",
 	"Marc Ewing",
-	N_("Gerg rdi"),
+	/* if your encoding allows it, use odoubleacute (U0151) for the 'o'
+	 * of 'Gergo' and use Eacute (U00C9) for the 'E' of 'Erdi' */
+	N_("Gergo Erdi"),
 	"Peter Fales",
 	"Joaquim Fellmann",
+	"Jens Finke",
 	"Dave Finton",
 	"Milon Firikis",
-	"Raul Perusquia Flores",
+	"John Fleck",
+	N_("Raul Perusquia Flores"),
 	"Lawrence Foard",
 	"Jeff Freedman",
 	"David Abilleira Freijeiro",
@@ -86,11 +93,13 @@ static gchar *contributors[] = {
 	"Jeff Garzik",
 	N_("The Mysterious GEGL"),
 	"Tim Gerla",
-	"Bjoern Giesler",
+	N_("Bjoern Giesler"),
 	"Dave Glowacki",
 	"Scott Goehring",
 	"Randy Gordon",
-	"Dov Grobgeld",
+	"Kenny Graunke",
+	"Jamin Philip Gray",
+	N_("Dov Grobgeld"),
 	"Bertrand Guiheneuf",
 	"Alan Aspuru Guzik",
 	"Telsa Gwynne",
@@ -115,8 +124,9 @@ static gchar *contributors[] = {
 	"Michael Kellen",
 	"Stephen Kiernan",
 	"Spencer Kimball",
+	"Alexander Kirillov",
 	"Peter Kirchgessner",
-	"Helmut Koeberle",
+	N_("Helmut Koeberle"),
 	"Alfredo Kojima",
 	"Andrew Kuchling",
 	"Stephan Kulow",
@@ -137,6 +147,7 @@ static gchar *contributors[] = {
 	"Garrett LeSage",
 	"Jason Leach",
 	"George Lebl",
+	"Gregory Leblanc",
 	"Elliot Lee",
 	"Marc Lehmann",
 	"Raph Levien",
@@ -149,7 +160,7 @@ static gchar *contributors[] = {
 	"Mandrake",
 	"Daniel Manrique",
 	"Kjartan Maraas",
-	"Matthew Marjanovic",
+	N_("Matthew Marjanovic"),
 	"Heath Martin",
 	"Oliver Maruhn",
 	"Dave Mason",
@@ -168,21 +179,26 @@ static gchar *contributors[] = {
 	"Jaka Mocnik",
 	"Paolo Molaro",
 	"David Mosberger",
+	"Dan Mueth",
 	"Thomas Muldowney",
-	/* for languages that can't display ntilde, replace '' with 'n' */
-	N_("Alexandre Muiz"),
+	/* If your encodings allows it, use ntilde (U00F1) for the 'n'
+	 * of 'Muniz' */
+	N_("Alexandre Muniz"),
+	/* Name in Hangeul is: Ub0a8 Uc131 Ud604 */
 	N_("Sung-Hyun Nam"),
 	"Karl Nelson",
 	"Asger Alstrup Nielsen",
 	"Eric Nielson",
-	/* languages that can't display adiaeresis, replace '' with 'ae' */
-	N_("Martin Norbck"),
+	/* If your encodings allows it, use adiaeresis (U00E4) for the 'ae'
+	 * of 'Norbaeck' */
+	N_("Martin Norbaeck"),
 	"Eskil Olsen",
 	"Jimmy Olsen",
 	"David Orme",
 	"Karl Anders Oygard",
-	/* languages that can't display Odiaeresis, replace '' with 'Oe' */
-	N_("Tomas gren"),
+	/* If your encodings allows it, use Odiaeresis (U00D6) for the 'Oe'
+	 * of 'Oegren' */
+	N_("Tomas Oegren"),
 	"Jay Painter",
 	"Cameron Parish",
 	"Conrad Parker",
@@ -199,6 +215,7 @@ static gchar *contributors[] = {
 	"Tero Pulkkinen",
 	"The Rasterman",
 	"Oliver Rauch",
+	"Christian Rose",
 	"Cody Russell",
 	"Reklaw",
 	"Jens Christian Restemeier",
@@ -211,18 +228,20 @@ static gchar *contributors[] = {
 	"Peter Ryland",
 	"Bibek Sahu",
 	"Pablo Saratxaga",
+	/* Name in Japanese is: U7b39 U5c71 U548c U5b8f */
 	N_("Kazuhiro Sasayama"),
 	"Carsten Schaar",
 	"Franck Schneider",
 	"Ingo Schneider",
 	"Bernd Schroeder",
 	"John Schulien",
-	/* for languages that can't display ntilde, replace '' with 'n' */ 
-	N_("Iigo Serna"),
+	/* If your encodings allows it, use ntilde (U00F1) for the 'n'
+	 * of 'Inigo' */
+	N_("Inigo Serna"),
 	"Shaleh",
 	"Joe \"Harold\" Shaw",
 	"Alejandro Aguilar Sierra",
-	"Miroslav Silovic",
+	N_("Miroslav Silovic"),
 	"Manish Singh",
 	"Timo Sirainen",
 	"David F. Skoll",
@@ -230,14 +249,14 @@ static gchar *contributors[] = {
 	"Josh Sled",
 	"John Slee",
 	"Garrett Smith",
-	/* not really a person name :) */
+	/* not really a person name :) translate its meaning */
 	N_("The Squeaky Rubber Gnome"),
 	"Maciej Stachowiak",
 	"Stalyn",
 	"Ben Stern",
 	"Micah Stetson",
 	"Nathan Carl Summers",
-	"Istvan Szekeres",
+	N_("Istvan Szekeres"),
 	"Tristan Tarrant",
 	"Anthony Taylor",
 	"Owen Taylor",
@@ -248,8 +267,8 @@ static gchar *contributors[] = {
 	"Christoph Toshok",
 	"Tom Tromey",
 	"Jon Trowbridge",
-	"Manish Vachharajani",
-	"Neil Vachharajani",
+	N_("Manish Vachharajani"),
+	N_("Neil Vachharajani"),
 	"Daniel Veillard",
 	"Vendu",
 	"Andrew Veliath",
