@@ -78,6 +78,7 @@ kill_applet_in_idle(gpointer data)
 	AppletInfo *info = data;
 	if (info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
+		extern_save_last_position (ext, TRUE /* sync */);
 		ext->clean_remove = TRUE;
 	} else if (info->type == APPLET_SWALLOW) {
 		Swallow *swallow = info->data;
@@ -120,6 +121,17 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			launcher_properties(menu->info->data);
 		else if (strcmp (menu->name, "help") == 0)
 			panel_show_help ("launchers.html");
+		else if (strcmp (menu->name, "help_on_app") == 0) {
+			Launcher * launcher = menu->info->data;
+			if (launcher->dentry != NULL) {
+				char *path = panel_gnome_kde_help_path
+					(launcher->dentry->docpath);
+				if (path != NULL) {
+					gnome_url_show (path);
+					g_free (path);
+				}
+			}
+		}
 		break;
 	case APPLET_DRAWER: 
 		if(strcmp(menu->name,"properties")==0) {
