@@ -695,6 +695,7 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 	char *title = NULL;
 	int slen;
 	GtkWidget *pixmap  = NULL;
+	char *name;
 
 	static GwmhDesk *desk = NULL;
 
@@ -702,16 +703,23 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 
 	if (GWMH_TASK_SKIP_WINLIST (task))
 		return;
-	/*g_message ("task: %s", task->name);*/
-	slen = strlen (task->name);
-	if (slen > 443)
-		title = g_strdup_printf ("%.420s...%s", task->name, task->name+slen-20);
+	if (task->name != NULL) {
+		slen = strlen (task->name);
+		if (slen > 443)
+			title = g_strdup_printf ("%.420s...%s", task->name, task->name+slen-20);
+		else
+			title = g_strdup (task->name);
+	} else {
+		/* Translators: Task with no name, should not really happen, so
+		 * this should signal that the panel is confused by this task
+		 * (thus question marks) */
+		title = g_strdup (_("???"));
+	}
 
 	if (GWMH_TASK_ICONIFIED (task)) {
 		char *tmp = title;
-		title = g_strdup_printf ("[%s]", title ? title : task->name);
-		if (tmp)
-			g_free (tmp);
+		title = g_strdup_printf ("[%s]", title);
+		g_free (tmp);
 	}
 	
 	item = gtk_pixmap_menu_item_new ();
@@ -722,7 +730,7 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 						 pixmap);
 	}
 
-	label = gtk_label_new (title ? title : task->name);
+	label = gtk_label_new (title);
 	g_free (title);
 
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -749,14 +757,25 @@ create_task_menu (GtkWidget *w, gpointer data)
 {
 	FoobarWidget *foo = FOOBAR_WIDGET (data);
 	GList *tasks = gwmh_task_list_get ();
+	GList *list;
+	GtkWidget *separator;
 
 	/*g_message ("creating...");*/
 	foo->tasks = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-	add_menu_separator (foo->task_menu);
+	separator = add_menu_separator (foo->task_menu);
 
 	g_list_foreach (tasks, (GFunc)add_task, foo);
 
+	list = g_list_last (GTK_MENU_SHELL (foo->task_menu)->children);
+
+	if (list != NULL &&
+	    separator == list->data) {
+		/* if the separator is the last item wipe it.
+		 * We leave it as the first though */
+		gtk_widget_destroy (separator);
+	}
+
 	/* Owen: don't read the next line */
 	GTK_MENU_SHELL (GTK_MENU_ITEM (w)->submenu)->active = 1;
 	our_gtk_menu_position (GTK_MENU (GTK_MENU_ITEM (w)->submenu));
@@ -820,12 +839,16 @@ set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
 	if (!GTK_WIDGET_REALIZED (foo))
 		return;
 
+	foo->icon_task = NULL;
+
 	if (foo->task_pixmap != NULL)
 		gtk_widget_destroy (foo->task_pixmap);
 	foo->task_pixmap = NULL;
 
-	if (task != NULL)
+	if (task != NULL) {
 		foo->task_pixmap = get_task_icon (task, GTK_WIDGET (foo));
+		foo->icon_task = task;
+	}
 
 	if (foo->task_pixmap == NULL) {
 		foo->task_pixmap = get_default_pixmap ();
@@ -838,7 +861,8 @@ set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
 }
 
 static gboolean
-task_notify (gpointer data, GwmhTask *task,
+task_notify (gpointer data,
+	     GwmhTask *task,
 	     GwmhTaskNotifyType ntype,
 	     GwmhTaskInfoMask imask)
 {
@@ -847,19 +871,32 @@ task_notify (gpointer data, GwmhTask *task,
 
 	switch (ntype) {
 	case GWMH_NOTIFY_INFO_CHANGED:
-		if (imask & GWMH_TASK_INFO_FOCUSED &&
-		    GWMH_TASK_FOCUSED (task)) {
-			set_das_pixmap (foo, task);
-		}
 		if (imask & GWMH_TASK_INFO_WM_HINTS &&
-		    GWMH_TASK_FOCUSED (task))
+		    GWMH_TASK_FOCUSED (task)) {
+			/* icon might have changed */
 			set_das_pixmap (foo, task);
+		} else if (imask & GWMH_TASK_INFO_FOCUSED) {
+			if (GWMH_TASK_FOCUSED (task) &&
+			    foo->icon_task != task) {
+				/* Focused and not set in the top thingie,
+				 * so setup */
+				set_das_pixmap (foo, task);
+			} else if ( ! GWMH_TASK_FOCUSED (task) &&
+				   task == foo->icon_task) {
+				/* Just un-focused and currently the
+				 * icon_task, so set the pixmap to
+				 * the default (nothing) */
+				set_das_pixmap (foo, NULL);
+			}
+		}
 		break;
 	case GWMH_NOTIFY_NEW:
 		if (foo->tasks != NULL)
 			add_task (task, foo);
 		break;
 	case GWMH_NOTIFY_DESTROY:
+		if (task == foo->icon_task)
+			set_das_pixmap (foo, NULL);
 		/* FIXME: Whoa; leak? */
 		if (foo->tasks != NULL) {
 			item = g_hash_table_lookup (foo->tasks, task);
@@ -868,7 +905,7 @@ task_notify (gpointer data, GwmhTask *task,
 				gtk_widget_hide (item);
 			} else {
 				g_warning ("Could not find item for task '%s'",
-					   task->name);
+					   sure_string (task->name));
 			}
 		}
 		break;
@@ -934,6 +971,12 @@ foobar_widget_init (FoobarWidget *foo)
 
 	foo->screen = 0;
 
+	foo->task_item = NULL;
+	foo->task_menu = NULL;
+	foo->task_pixmap = NULL;
+	foo->task_bin = NULL;
+	foo->icon_task = NULL;
+
 	foo->clock_format = g_strdup (_("%H:%M"));
 	foo->clock_timeout = 0;
 	foo->clock_label = NULL;
