@@ -596,12 +596,14 @@ clean_kill_applets (PanelWidget *panel)
 					     "applet_info");
 		if (info->type == APPLET_EXTERN) {
 			Extern *ext = info->data;
+			extern_save_last_position (ext, FALSE /* sync */);
 			ext->clean_remove = TRUE;
 		} else if (info->type == APPLET_SWALLOW) {
 			Swallow *swallow = info->data;
 			swallow->clean_remove = TRUE;
 		}
 	}
+	gnome_config_sync ();
 }
 
 static void
@@ -643,6 +645,8 @@ panel_destroy (GtkWidget *widget, gpointer data)
 		gtk_widget_unref (pd->menu);
 	pd->menu = NULL;
 
+	pd->panel = NULL;
+
 	panel_list = g_slist_remove (panel_list, pd);
 	g_free (pd);
 }
@@ -938,12 +942,16 @@ extract_filename (const gchar* uri)
 }
 
 static void
-drop_url(PanelWidget *panel, int pos, char *url)
+drop_url(PanelWidget *panel, int pos, const char *url)
 {
-	char *p = g_strdup_printf (_("Open URL: %s"), url);
+	char *p;
+
+	g_return_if_fail (url != NULL);
+
+	p = g_strdup_printf (_("Open URL: %s"), url);
 	load_launcher_applet_from_info_url (url, p, url, "gnome-globe.png",
 					    panel, pos, TRUE);
-	g_free(p);
+	g_free (p);
 }
 
 static void
@@ -961,6 +969,68 @@ drop_menu (PanelWidget *panel, int pos, const char *dir)
 	load_menu_applet (dir, flags, TRUE, FALSE, NULL, panel, pos, TRUE);
 }
 
+static void
+drop_directory (PanelWidget *panel, int pos, const char *dir)
+{
+	char *tmp;
+
+	tmp = g_concat_dir_and_file (dir, ".directory");
+	if (panel_file_exists (tmp)) {
+		g_free (tmp);
+		drop_menu (panel, pos, dir);
+		return;
+	}
+	g_free (tmp);
+
+	tmp = g_concat_dir_and_file (dir, ".order");
+	if (panel_file_exists (tmp)) {
+		g_free (tmp);
+		drop_menu (panel, pos, dir);
+		return;
+	}
+	g_free (tmp);
+
+	tmp = panel_is_program_in_path ("nautilus");
+	if (tmp != NULL) {
+		/* nautilus */
+		char *exec[] = { "nautilus", (char *)dir, NULL };
+
+		g_free (tmp);
+
+		load_launcher_applet_from_info (g_basename (dir),
+						dir,
+						exec,
+						2,
+						"gnome-folder.png",
+						panel,
+						pos,
+						TRUE);
+	} else {
+		tmp = panel_is_program_in_path ("gmc-client");
+		if (tmp != NULL) {
+			/* gmc */
+			char *exec[] = {
+				"gmc-client",
+				"--create-directory",
+				(char *)dir,
+				NULL };
+
+			g_free (tmp);
+
+			load_launcher_applet_from_info (g_basename (dir),
+							dir,
+							exec,
+							3,
+							"gnome-folder.png",
+							panel,
+							pos,
+							TRUE);
+		} else {
+			drop_menu (panel, pos, dir);
+		}
+	}
+}
+
 static void
 drop_urilist (PanelWidget *panel, int pos, char *urilist,
 	      gboolean background_drops)
@@ -1013,7 +1083,7 @@ drop_urilist (PanelWidget *panel, int pos, char *urilist,
 			if (launcher != NULL)
 				launcher_hoard (launcher);
 		} else if (S_ISDIR(s.st_mode)) {
-			drop_menu(panel, pos, filename);
+			drop_directory (panel, pos, filename);
 		} else if (S_IEXEC & s.st_mode) /*executable?*/
 			ask_about_launcher (filename, panel, pos, TRUE);
 		g_free (filename);
@@ -1392,7 +1462,7 @@ drag_data_recieved_cb (GtkWidget	*widget,
 			drop_bgimage (panel, (char *)selection_data->data);
 		break;
 	case TARGET_DIRECTORY:
-		drop_menu (panel, pos, (char *)selection_data->data);
+		drop_directory (panel, pos, (char *)selection_data->data);
 		break;
 	case TARGET_APPLET:
 		if ( ! selection_data->data) {
@@ -1678,3 +1748,26 @@ panel_set_id (GtkWidget *widget, int id)
 	else if (IS_FOOBAR_WIDGET (widget))
 		PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel)->unique_id = id;
 }
+
+void
+status_unparent (GtkWidget *widget)
+{
+	GList *li;
+	PanelWidget *panel = NULL;
+	if (IS_BASEP_WIDGET (widget))
+		panel = PANEL_WIDGET(BASEP_WIDGET(widget)->panel);
+	else if (IS_FOOBAR_WIDGET (widget))
+		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
+	for(li=panel->applet_list;li;li=li->next) {
+		AppletData *ad = li->data;
+		AppletInfo *info = gtk_object_get_data(GTK_OBJECT(ad->applet),
+						       "applet_info");
+		if(info->type == APPLET_STATUS) {
+			status_applet_put_offscreen(info->data);
+		} else if(info->type == APPLET_DRAWER) {
+			Drawer *dr = info->data;
+			status_unparent(dr->drawer);
+		}
+	}
+}
+
